#include "sceneprimitive.h"
#include "algorithm"

ScenePrimitive::ScenePrimitive(PrimitiveType type, int tesselation, QObject *parent)
    :SceneObject(parent)
{
    this->type = type;
    this->tesselation = tesselation;

    qobj = gluNewQuadric();
    gluQuadricNormals(qobj, GLU_SMOOTH);

    this->color = new float[3];
    this->color[0] = 0.0;
    this->color[1] = 0.0;
    this->color[2] = 0.0;
}

void ScenePrimitive::draw(){
    glPushMatrix();
    applyTransformation();

    glMaterialfv(GL_FRONT,GL_AMBIENT,color);
    GLfloat white[] = {1.0,1.0,1.0};
    glMaterialfv(GL_FRONT,GL_DIFFUSE,color);
    glMaterialfv(GL_FRONT,GL_SPECULAR,white);

    GLfloat mShininess[] = {128};
    glMaterialfv(GL_FRONT,GL_SHININESS,mShininess);

    int radius = 1;

    switch (this->type) {
    case Quader:
        drawCube(tesselation);
        break;
    case Sphere:
        gluSphere(qobj,1,tesselation,tesselation);
        break;
    case Cylinder:

        gluCylinder(qobj,radius,radius,3,tesselation,tesselation);
        gluQuadricOrientation(qobj,GLU_INSIDE);
        gluDisk( qobj, 0.0, radius, tesselation, 1);
        glTranslatef(0,0,3);
        gluQuadricOrientation(qobj,GLU_OUTSIDE);
        gluDisk( qobj, 0.0 , radius, tesselation, 1);
        break;
    case Torus:
      //  glutSolidTorus(5, 15, 8, 128);
        break;
    default:
        qDebug()<<"Enum Error";
        break;
    }


    glPopMatrix();
}

void ScenePrimitive::setMaterial(float *color )
{


    this->color = new float[3];
    this->color[0] = color[0];
    this->color[1] = color[1];
    this->color[2] = color[2];
   // qDebug()<< *this->color<<" "<<this->color[0]<<" "<<this->color[1]<<" "<<this->color[2];

}


void ScenePrimitive::drawCube(int tesselation)
{
    float increment = 1.0/(pow(2,tesselation));

    glBegin(GL_QUADS);
    glNormal3f(0,1,0);
    for (float x = -0.5; x < 0.5 ; x+= increment) {
        for (float y = -0.5; y < 0.5 ; y+= increment ) {
            glVertex3f( x+increment, 0.5f,y);
            glVertex3f(x, 0.5f,y);
            glVertex3f(x, 0.5f, y+increment);
            glVertex3f( x+increment, 0.5f, y+increment);
        }
    }
    glEnd();

    glBegin(GL_QUADS);
    glNormal3f(0,-1,0);
    for (float x = -0.5; x < 0.5 ; x+= increment) {
        for (float y = -0.5; y < 0.5 ; y+= increment ) {
            glVertex3f( x+increment, -0.5f,y+increment);
            glVertex3f(x, -0.5f,y+increment);
            glVertex3f(x, -0.5f, y);
            glVertex3f( x+increment, -0.5f, y);
        }
    }
    glEnd();

    glBegin(GL_QUADS);
    glNormal3f(0,0,1);
    for (float x = -0.5; x < 0.5 ; x+= increment) {
        for (float y = -0.5; y < 0.5 ; y+= increment ) {
            glVertex3f( x+increment,y+increment, 0.5f);
            glVertex3f(x, y+increment, 0.5f);
            glVertex3f(x,  y, 0.5f);
            glVertex3f( x+increment, y, 0.5f);
        }
    }
    glEnd();

    glBegin(GL_QUADS);
    glNormal3f(0,0,-1);
    for (float x = -0.5; x < 0.5 ; x+= increment) {
        for (float y = -0.5; y < 0.5 ; y+= increment ) {
            glVertex3f( x+increment,y, -0.5f);
            glVertex3f(x, y,-0.5f);
            glVertex3f(x, y+increment,-0.5f);
            glVertex3f( x+increment,y+increment, -0.5f);
        }
    }
    glEnd();

    glBegin(GL_QUADS);
    glNormal3f(-1,0,0);
    for (float x = -0.5; x < 0.5 ; x+= increment) {
        for (float y = -0.5; y < 0.5 ; y+= increment ) {
            glVertex3f( -0.5f, x+increment,y+increment);
            glVertex3f( -0.5f,x+increment,y);
            glVertex3f( -0.5f,x, y);
            glVertex3f(  -0.5f,x, y+increment);
        }
    }
    glEnd();

    glBegin(GL_QUADS);
    glNormal3f(1,0,0);
    for (float x = -0.5; x < 0.5 ; x+= increment) {
        for (float y = -0.5; y < 0.5 ; y+= increment ) {
            glVertex3f( 0.5f, x+increment,y);
            glVertex3f( 0.5f,x+increment,y+increment);
            glVertex3f( 0.5f,x, y+increment);
            glVertex3f(  0.5f,x, y);
        }
    }
    glEnd();
}

